"""
Hand Tracking Module
By : JikanDev
Website : https://jikandev.xyz/
"""
import cv2
import mediapipe as mp


class HandDetector:
    """
    Finds Hands using the mediapipe library. Exports the landmarks in pixel format.
    Also provides bounding box info of the hand found.
    """

    def __init__(self, mode=False, maxHands=2, minDetectCon=0.5, minTrackCon=0.5):
        """
        :param mode: In static mode, detection is done on each image: slower.
        :param maxHands: Maximum number of hands to detect.
        :param minDetectCon: Minimum Detection Confidence Threshold.
        :param minTrackCon: Minimum Tracking Confidence Threshold.
        """
        self.mode = mode
        self.maxHands = maxHands
        self.minDetectCon = minDetectCon
        self.minTrackCon = minTrackCon

        self.mpHands = mp.solutions.hands
        self.hands = self.mpHands.Hands(self.mode, self.maxHands, self.minDetectCon, self.minTrackCon)
        self.mpDraw = mp.solutions.drawing_utils

        self.lmList = []

    def findHands(self, img, draw=True, drawBboxs=True, camFlip=True):
        """
        Finds hands in a BGR image.
        :param img: Image to find the hands in.
        :param draw: Flag to draw the output on the image.
        :param drawBboxs: Flag to draw bboxs on the draw output.
        :param camFlip: Flag to know if your camera flip your image.
        :return: Image with or without drawings.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.hands.process(imgRGB)

        allHands = []
        h, w, c = img.shape

        if self.results.multi_hand_landmarks:
            for handType, handLms in zip(self.results.multi_handedness, self.results.multi_hand_landmarks):
                myHand = {}

                mylmList = []
                xList = []
                yList = []

                for id, lm in enumerate(handLms.landmark):
                    px, py = int(lm.x * w), int(lm.y * h)
                    mylmList.append([px, py])
                    xList.append(px)
                    yList.append(py)

                xmin, xmax = min(xList), max(xList)
                ymin, ymax = min(yList), max(yList)
                boxW, boxH = xmax - xmin, ymax - ymin
                bbox = xmin, ymin, boxW, boxH
                cx, cy = bbox[0] + (bbox[2] // 2), bbox[1] + (bbox[3] // 2)

                myHand["lmList"] = mylmList
                myHand["bbox"] = bbox
                myHand["center"] = (cx, cy)

                if camFlip:
                    if handType.classification[0].label == "Right":
                        myHand["type"] = "Left"
                    else:
                        myHand["type"] = "Right"
                else:
                    myHand["type"] = handType.classification[0].label
                allHands.append(myHand)

                if draw:
                    self.mpDraw.draw_landmarks(img, handLms, self.mpHands.HAND_CONNECTIONS)
                    if drawBboxs:
                        cv2.rectangle(img, (bbox[0] - 20, bbox[1] - 20), (bbox[0] + bbox[2] + 20, bbox[1] + bbox[3] + 20), (255, 255, 255), 2)
                        cv2.putText(img, myHand["type"], (bbox[0] - 30, bbox[1] - 30), cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)

        return allHands, img


def main():
    """
    Example code to use the module.
    """
    cap = cv2.VideoCapture(0)  # Get your camera
    detector = HandDetector()  # Call the HandDetector class

    while True:
        success, img = cap.read()  # If success, img = read your camera image
        hands, img = detector.findHands(img)  # hands & img call the findHands() function of HandDetector

        if hands:
            # Hand 1
            hand1 = hands[0]
            lmList1 = hand1["lmList"]  # List of 21 Landmark points
            bbox1 = hand1["bbox"]  # Bounding box info x,y,w,h
            centerPoint1 = hand1['center']  # Center of the hand cx,cy
            handType1 = hand1["type"]  # Hand Type "Left" or "Right"

            if len(hands) == 2:
                # Hand 2
                hand2 = hands[1]
                lmList2 = hand2["lmList"]  # List of 21 Landmark points
                bbox2 = hand2["bbox"]  # Bounding box info x,y,w,h
                centerPoint2 = hand2['center']  # Center of the hand cx,cy
                handType2 = hand2["type"]  # Hand Type "Left" or "Right"

        cv2.imshow("Hands Tracking Module", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()


"""
Face Detection Module
By : JikanDev
Website : https://jikandev.xyz/
"""
import cv2
import mediapipe as mp


class FaceDetector():
    """
    Find faces in realtime using the light weight model provided in the mediapipe library.
    """
    def __init__(self, minDetectCon=0.5, model=0):
        """
        :param minDetectCon: Minimum Detection Confidence Threshold.
        :param model: Model adaptation, 0 to select a short-range model that works
        best for faces within 2 meters from the camera, and 1 for a full-range
        model best for faces within 5 meters.
        """
        self.minDetectCon = minDetectCon
        self.model = model

        self.mpFaceDetection = mp.solutions.face_detection
        self.mpDraw = mp.solutions.drawing_utils
        self.faceDetection = self.mpFaceDetection.FaceDetection(self.minDetectCon, self.model)

    def findFaces(self, img, draw=True):
        """
        Find faces in an image and return the bbox info
        :param img: Image to find the faces in.
        :param draw: Flag to draw the output on the image.
        :return: Image with or without drawings.
                 Bounding Box list.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.faceDetection.process(imgRGB)

        bboxs = []
        if self.results.detections:
            for id, detection in enumerate(self.results.detections):
                bboxC = detection.location_data.relative_bounding_box
                ih, iw, ic = img.shape
                bbox = int(bboxC.xmin * iw), int(bboxC.ymin * ih), int(bboxC.width * iw), int(bboxC.height * ih)
                bboxs.append([id, bbox, detection.score])
                if draw:
                    img = self.fancyDraw(img,bbox)
                    cv2.putText(img, f'{int(detection.score[0] * 100)}%', (bbox[0], bbox[1] - 20), cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)

        return img, bboxs

    def fancyDraw(self, img, bbox, l=30, t=5, rt= 1):
        x, y, w, h = bbox
        x1, y1 = x + w, y + h

        cv2.rectangle(img, bbox, (255, 255, 255), rt)

        cv2.line(img, (x, y), (x + l, y), (0, 0, 255), t)
        cv2.line(img, (x, y), (x, y+l), (0, 0, 255), t)

        cv2.line(img, (x1, y), (x1 - l, y), (0, 0, 255), t)
        cv2.line(img, (x1, y), (x1, y+l), (0, 0, 255), t)

        cv2.line(img, (x, y1), (x + l, y1), (0, 0, 255), t)
        cv2.line(img, (x, y1), (x, y1 - l), (0, 0, 255), t)

        cv2.line(img, (x1, y1), (x1 - l, y1), (0, 0, 255), t)
        cv2.line(img, (x1, y1), (x1, y1 - l), (0, 0, 255), t)
        return img


def main():
    """
    Example code to use the module.
    """
    cap = cv2.VideoCapture(0)  # Get your camera
    detector = FaceDetector()  # Call the FaceDetector class

    while True:
        success, img = cap.read()  # If success, img = read your camera image
        img, bboxs = detector.findFaces(img)  # img & bboxs call the findFaces() function of FaceDetector

        cv2.imshow("Face Detection Module", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()


"""
Body Tracking Module
By : JikanDev
Website : https://jikandev.xyz/
"""
import cv2
import mediapipe as mp


class BodyDetector:
    """
    Finds Bodies using the mediapipe library. Exports the landmarks in pixel format.
    Also provides bounding box info of the body found.
    """

    def __init__(self, mode=False, model=1, smooth_lm=True, segmentation=False, smooth_segmentation=True,
                 minDetectCon=0.5, minTrackCon=0.5):
        """
        :param mode: In static mode, detection is done on each image: slower.
        :param model: Complexity of the pose landmark model: 0, 1 or 2. See
        details in https://solutions.mediapipe.dev/pose#model_complexity.
        :param smooth_lm: Whether to filter landmarks across different input
        images to reduce jitter.
        :param segmentation: Whether to predict segmentation mask.
        :param smooth_segmentation: Whether to filter segmentation across different input
        images to reduce jitter.
        :param minDetectCon: Minimum Detection Confidence Threshold.
        :param minTrackCon: Minimum Tracking Confidence Threshold.
        """
        self.mode = mode
        self.model = model
        self.smooth_lm = smooth_lm
        self.segmentation = segmentation
        self.smooth_segmentation = smooth_segmentation
        self.minDetectCon = minDetectCon
        self.minTrackCon = minTrackCon

        self.mpPose = mp.solutions.pose
        self.pose = self.mpPose.Pose(self.mode, self.model, self.smooth_lm, self.segmentation, self.smooth_segmentation,
                                     self.minDetectCon, self.minTrackCon)
        self.mpDraw = mp.solutions.drawing_utils

        self.lmList = []

    def findBodies(self, img, draw=True, drawBboxs=False):
        """
        Finds bodies in a BGR image.
        :param img: Image to find the hands in.
        :param draw: Flag to draw the output on the image.
        :param drawBboxs: Flag to draw bboxs on the draw output.
        :return: Image with or without drawings.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.pose.process(imgRGB)

        allBodies = []
        h, w, c = img.shape

        if self.results.pose_landmarks:
            bodyLms = self.results.pose_landmarks

            myBody = {}

            mylmList = []
            xList = []
            yList = []

            for id, lm in enumerate(bodyLms.landmark):
                px, py = int(lm.x * w), int(lm.y * h)
                mylmList.append([px, py])
                xList.append(px)
                yList.append(py)

            xmin, xmax = min(xList), max(xList)
            ymin, ymax = min(yList), max(yList)
            boxW, boxH = xmax - xmin, ymax - ymin
            bbox = xmin, ymin, boxW, boxH
            cx, cy = bbox[0] + (bbox[2] // 2), bbox[1] + (bbox[3] // 2)

            myBody["lmList"] = mylmList
            myBody["bbox"] = bbox
            myBody["center"] = (cx, cy)

            allBodies.append(myBody)

            if draw:
                self.mpDraw.draw_landmarks(img, self.results.pose_landmarks, self.mpPose.POSE_CONNECTIONS)
                if drawBboxs:
                    cv2.rectangle(img, (bbox[0] - 20, bbox[1] - 100), (bbox[0] + bbox[2] + 20, bbox[1] + bbox[3] + 20), (255, 255, 255), 2)
                    cv2.putText(img, "Body", (bbox[0] - 30, bbox[1] - 110), cv2.FONT_HERSHEY_PLAIN, 2, (255, 255, 255), 2)

        return allBodies, img


def main():
    """
    Example code to use the module.
    """
    cap = cv2.VideoCapture(0)  # Get your camera
    detector = BodyDetector()  # Call the BodyDetector class

    while True:
        success, img = cap.read()  # If success, img = read your camera image
        bodies, img = detector.findBodies(img)  # bodies & img call the findBodies() function of BodyDetector

        if bodies:
            # Body 1
            body1 = bodies[0]
            lmList1 = body1["lmList"]  # List of 21 Landmark points
            bbox1 = body1["bbox"]  # Bounding box info x,y,w,h
            centerPoint1 = body1['center']  # Center of the body cx,cy

            if len(bodies) == 2:
                # Body 2
                body2 = bodies[1]
                lmList2 = body2["lmList"]  # List of 21 Landmark points
                bbox2 = body2["bbox"]  # Bounding box info x,y,w,h
                centerPoint2 = body2['center']  # Center of the body cx,cy

        cv2.imshow("Body Tracking Module", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()


from jikanvision.HandsTrackingModule import HandDetector
from jikanvision.FaceDetectionModule import FaceDetector
from jikanvision.BodyTrackingModule import BodyDetector
from jikanvision.FaceMeshModule import FaceMeshDetector


"""
Face Mesh Module
By : JikanDev
Website : https://jikandev.xyz/
"""
import cv2
import mediapipe as mp


class FaceMeshDetector():
    """
    Find 468 Landmarks using the mediapipe library. Exports the landmarks in pixel format.
    """

    def __init__(self, mode=False, maxFaces=1, refine_lm=False, minDetectCon=0.5, minTrackCon=0.5):
        """
        :param mode: In static mode, detection is done on each image: slower.
        :param maxFaces: Maximum number of faces to detect.
        :param refine_lm: Whether to further refine the landmark coordinates
        around the eyes and lips, and output additional landmarks around the
        irises.
        :param minDetectCon: Minimum Detection Confidence Threshold.
        :param minTrackCon: Minimum Tracking Confidence Threshold.
        """
        self.mode = mode
        self.maxFaces = maxFaces
        self.refine_lm = refine_lm
        self.minDetectCon = minDetectCon
        self.minTrackCon = minTrackCon

        self.mpDraw = mp.solutions.drawing_utils
        self.mpDrawingStyles = mp.solutions.drawing_styles
        self.faceMesh = mp.solutions.face_mesh

        self.meshDetection = self.faceMesh.FaceMesh(mode, maxFaces, refine_lm, minDetectCon, minTrackCon)

    def findFaces(self, img, draw=True, drawTesselation=True):
        """
        Find faces in an image and return the bbox info
        :param img: Image to find the faces in.
        :param draw: Flag to draw the output contours of the mesh on the image.
        :param drawTesselation: Flag to draw the output tesselation of the mesh on the image.
        :return: Image with or without drawings.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.meshDetection.process(imgRGB)

        allFaces = []
        h, w, c = img.shape

        if self.results.multi_face_landmarks:
            for faceLms in self.results.multi_face_landmarks:
                myMesh = {}
                mylmList = []

                for id, lm in enumerate(faceLms.landmark):
                    px, py = int(lm.x * w), int(lm.y * h)
                    mylmList.append([px, py])

                myMesh["lmList"] = mylmList

                if draw:
                    self.mpDraw.draw_landmarks(img, faceLms, self.faceMesh.FACEMESH_CONTOURS, None)

                if drawTesselation:
                    self.mpDraw.draw_landmarks(img, faceLms, self.faceMesh.FACEMESH_TESSELATION, None,
                                               self.mpDrawingStyles.get_default_face_mesh_tesselation_style())

                allFaces.append(myMesh)

        return allFaces, img


def main():
    """
    Example code to use the module.
    """
    cap = cv2.VideoCapture(0)  # Get your camera
    detector = FaceMeshDetector()  # Call the FaceMeshDetector class

    while True:
        success, img = cap.read()  # If success, img = read your camera image
        meshes, img = detector.findFaces(img)  # meshes & img call the findFaces() function of FaceMeshDetector

        if meshes:
            # Mesh 1
            mesh1 = meshes[0]
            lmList1 = mesh1["lmList"]  # List of 21 Landmark points

            if len(meshes) == 2:
                # Mesh 2
                mesh2 = meshes[1]
                lmList2 = mesh2["lmList"]  # List of 21 Landmark points

        cv2.imshow("Face Mesh Module", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()


from distutils.core import setup

setup(
  name = 'jikanvision',         # How you named your package folder (MyLib)
  packages = ['jikanvision'],   # Chose the same as "name"
  version = '0.5',      # Start with a small number and increase it with every change you make
  license='Apache License 2.0',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'My library for AI Computer Vision based on Mediapipe and OpenCV',   # Give a short description about your library
  author = 'JikanDev',                   # Type in your name
  author_email = 'jikandev@gmail.com',      # Type in your E-Mail
  url = 'https://jikandev.xyz/',   # Provide either the link to your github or to your website
  project_urls={
    "Bug Tracker": "https://github.com/JikanDev/jikanvision/issues",
  },
  keywords = ['Computer Vision', 'Face Detection', 'Face Mesh', 'Hands Tracking', 'Bodies Tracking'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'opencv-python==4.5.3.56',
          'mediapipe==0.8.8',
      ],
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: Apache Software License',   # Again, pick a license.
    'Programming Language :: Python :: 3.6',    #Specify which python versions that you want to support.
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)